/*  Copyright (C) 2017 Nicholas C. Strauss (strauss@positive-internet.com)
    This file is part of ORBTIMES.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <math.h>
#include <stdio.h>
#include "startime.h"
#include "angle.h"
#include "sun.h"
#include "moon2.h"
#include "moon3.h"
#include "equationOfTime.h"

/*
   jd = julian date for new moon (JD1900 , julian days since 1900)
   ni,nf = new moon integer and fractional parts.
   input: day,month,year 
   [full = 1 full moon] [full = 0, new moon]
   output: date of moon event in JD1900 (julian days since 1900)
 */
extended newmoon(extended day, extended month, extended year, short full, 
		 extended &argumentLatitude,
		 extended &eventDay,
		 extended &eventFraction
		 )
{
  extended j0, dj, k1, k, t, a1, a2,
    f, dd, c, b, e, e1, b1, ni, nf; 

  dj = JulianDays( day, month, year);
  j0 = JulianDays( 0, 1, year);         

  k1 = (year - 1900 + (dj - j0)/365.0 )*12.3685 ;
  if (!full) k = floor(k1+0.5) ;        //new moon
  else       k = floor(k1+0.5) + 0.5 ;  //full moon
  t = k/1236.85;

  a1 = 359.2242 + 360*FRAC(k/1.236886e1) - (3.33e-5 - 3.47e-6*t)*t*t;
  a2 = 306.0253 + 360*FRAC(k/9.330851e-1) + (1.07306e-2*t*t) + (1.236e-5*t*t*t);
  f  = 21.2964 + 360*FRAC(k/9.214926e-1)  - (1.6528e-3 - 2.39e-6*t)*t*t;
  a1 = deg2rad(fmod(a1,360));
  a2 = deg2rad(fmod(a2,360));
  f  = deg2rad(fmod(f,360));
  dd=(1.734e-1 - 3.9e-4*t)*sin(a1)+ 2.1e-3*sin(2*a1)            ;
  dd+= - 4.068e-1*sin(a2) + 1.61e-2*sin(2*a2) - 4e-4*sin(3*a2)           ;
  dd+= + 1.04e-2*sin(2*f) - 5.1e-3*sin(a1+a2)           ;
  dd+= - 7.4e-3*sin(a1-a2) + 4e-4*sin(2*f+a1)           ;
  dd+= - 4e-4*sin(2*f-a1) - 6e-4*sin(2*f+a2) + 1e-3*sin(2*f-a2)           ;
  dd+= + 5e-4*sin(a1+2*a2)           ;

  c  = deg2rad(166.56 + (132.87 - 9.173e-3*t) *t )         ;
  e  = (29.53) * k           ;
  e1 = rint ( e )           ;
  b  = dd + e - e1 +  5.8868e-4 * k  + (1.178e-4 - 1.55e-7 * t)*t*t
    + 3.3e-4 * sin(c) + 7.5933e-1           ;

  b1 = rint ( b );
  eventDay = e1 + b1;             // or fi
  eventFraction = b - b1;             // or ff
  argumentLatitude = f;
 return b + e1; // ni + nf;
}

#if 0
/*  calculate new (or full /option full) moon nearest (day,month,year) 
 *  with return in julian time (jd) and latitude (distance from the ecliptic node)
 *  radian units
 */
void newmoonString(char* str, extended day, extended month, extended year, short full, 
		   extended &latitude, julianWithDate &jwd)
{
  extended d, m, y, dow;	

  jwd.jd = newmoon(day, month, year, full, latitude, jwd.nat, jwd.frac);
  CalendarDays( jwd.jd, d, m, y, dow);

  dow = floor(dow);
  if (dow > 7 || dow < 1) dow = 7;
  else dow -= 1;
  m = floor(m);
  if (m > 12 || m < 1) m = 12;
  else m -= 1;
  sprintf(str,"%s \t%s \t%3.0f, %4.0f",
	  dayStringArray[(int)dow], monthStringArray[(int)m], 
	  (float)d, (float)y);
  jwd.dow = dow;
  jwd.day = d;
  jwd.month = m;
  jwd.year = y;
}
#endif

const char *
eclipse_check(extended moon_lat, extended &certain)
{
  moon_lat = fabs(moon_lat-rint(moon_lat/M_PI)*M_PI);
  if (moon_lat > 0.37) moon_lat = M_PI-moon_lat;
  if (moon_lat < 2.42600766e-1){
    certain = 1.0;
    return " certain eclipse";
  }
  else if (moon_lat < 0.37){
    certain = 0.5; 
    return " possible eclipse";
  }
  else {
    certain = 0.0;
    return "";
  }
}

/* Is there a solar eclipse on iday,imonth,iyear?
 * gl,gp,ht = lon,lat,elevation
 * Return 1 if eclipse, 0 otherwise
 * 
 * Based on BASIC routine 7000 ECLIPSE
 * from Peter Duffett-Smith's Astronomy with your Personal Computer
 * Thu Aug  3 18:16:57 PDT 2017
 * My motivation is the July 21, 2017 Solar Eclipse.
 * work-in-progress under construction.
 */ 
int
solar_eclipse(double gl, double gp, double ht, int iday, int imonth, int iyear)
{
  const int fullmoon = 0;  // 0 signifies new moon
  const int samples = 2;
  double   look = 1;
  extended eclipse_certain, hours_ut, hours_sun_A;
  extended event_day, event_fraction, argument_latitude;
  //  extended sr[samples], rr[samples], d2[samples], d3[samples], am[samples];  
  //  int i;
  //  extended  mm[samples], bm[samples], pm[samples], distance_to_moon[samples], phase, solar_longitude, solar_radius;
  double ly, my, by, hy, lz, mz, bz, hz, sb;
  double distance_to_moon, phase, solar_longitude, solar_radius;
  double rr, d2, d3, am;
  double sr, hp, zh, tc, xo, sh, s2, z2, ps, z1, rm, rn, hd, ru, rp, pj, r, dd;
  double xh, x0, dm, mr, dp, p3;
  double lj, h0;
  //  iday = 21;
  //  imonth = 8;
  //  iyear = 2017;
  // Check for new moon. 
  newmoon(iday, imonth, iyear, fullmoon, argument_latitude, event_day, event_fraction);

  // Check the argument_latitude for eclipse.
  eclipse_check(argument_latitude, eclipse_certain);
  if (fabs(1.-eclipse_certain)>0.1) return 0;

  hours_ut = (event_fraction-0.5)*24;

  if (hours_ut>24) hours_ut -= 24;
  if (hours_ut<0) hours_ut += 24;

  printf("hours_ut %lf\n", hours_ut);

  // Calculate Sun and Moon position -1hr,+1hr event.
  // -1hr
  //  ly sol lon
  //  my moon lon
  //  by moon lat
  //  hy moon hor parallax
  // +1hr
  //  sb = sol lon diff (sr-ly)
  //  mz moon lon
  //  bz moon lat
  //  hz moon hparallax
  // correct moon terms for local parallax

  double tzi = 0, ds = 0;
  double solar_lon, ecliptic_lat, ecliptic_lon, ra, dec, value_min, value_max, az, el, sun_az, sun_alt;
  double dj00 = floor(event_day);

  // for (look=0;look<1.;look+=.01){
  //   compute_moon_az_el(dj00, hours_ut+look, tzi, ds, gl, gp, solar_lon, ecliptic_lat, ecliptic_lon,
  // 		       ra, dec, value_min, value_max, az, el, phase, distance_to_moon);
  //   sun_alt_az(dj00, hours_ut+look, tzi, ds, gp, gl, 0, sun_az, sun_alt);

  //   printf("ut %lf moon az %lf el %lf sun az %lf el %lf\n", hours_ut+look, az, el, sun_az, sun_alt);

  double  dj=JulianDays(21,8,2017);
  hours_ut = 18.303667;
  hours_ut = 18.363667;
  sun_position(dj, hours_ut-look, ly, rr, d2, d3, am);
  calculate_moon(dj, hours_ut-look, my, by, hy, distance_to_moon, phase, solar_longitude, solar_radius, true);
  printf("ut %lf lat %lf lon %lf solar lon= %lf\n", hours_ut-look, by, my, ly);

  sun_position(dj, hours_ut+look, lz, rr, d2, d3, am);
  calculate_moon(dj, hours_ut+look, mz, bz, hz, distance_to_moon, phase, solar_longitude, solar_radius, true);
  //  compute_parallax(double &p, double &q, double x, double y, double gl, double gp, double dj, double ut, double ht, double hp);
  //  printf("sol ly %lf lz %lf\n", ly, lz);
  printf("ut %lf lat %lf lon %lf solar lon= %lf\n", hours_ut+look, bz, mz, lz);

  // Calculate time of lunar latitude = 0, when moon crosses the ecliptic, point A
  // interp lunar latitudes (by, bz) hours per latitude, over two hours
  //  xh = hours_ut;
  by-=0.422;
  bz-=0.422;
  x0 = hours_ut - 2 * bz/(bz-by);
  printf("Moon crosses ecliptic at %lf hours_ut =%lf\n", x0, hours_ut);
  calculate_moon(dj, x0, mz, bz, hz, distance_to_moon, phase, solar_longitude, solar_radius, true);
  printf("\tcrossing %lf %lf\n", bz, mz);

  return 0;
  // Calculate difference of lunar longitude (-1, +1 hrs)
  dm = mz-my;
  lj = (dm-sb)/2;

  // Lunar lon at point A
  mr = my + dm * (x0-xh+1.)/2.;

  // Time of Sun at point A, account for light speed, in hours ut
  hours_sun_A = x0-1.3851852e-1;

  // Calculate Sun position at point A.
  sun_position(event_day, hours_sun_A, sr, rr, d2, d3, am);
  printf("Sun A %lf lon=%lf dist=%lf d2=%lf d3=%lf am=%lf\n", hours_sun_A, sr, rr, d2, d3, am);

  //  sun_position();
  sr = sr + M_PI/180.*(dp-5.69e-3); //aberation
  // correct for parallax
  hp = 4.263452e-5/rr;

  // eclipse geometry
  zh = (sr-mr)/lj;
  tc = x0 + zh;
  sh = (((bz-by)*(tc-xh-1.)/2.)+bz)/lj;
  s2 = sh*sh;
  z2 = zh*zh;
  ps = p3/(rr*lj);
  z1 =zh*z2/(z2+s2)+x0;
  h0 =(hy+hz)/(2.*lj);
  rm = 2.72446e-1*h0;
  rn = 4.65242e-3/(lj*rr);
  hd = h0*0.99834;
  ru = (hd-rn+ps)*1.02; //umbral radius
  rp = (hd+rn+ps)*1.02; //penumbral radius
  pj = fabs(sh*zh)/((s2*z2)*(s2*z2));
  r = rm+rn;
  dd = z1-x0;
  dd = dd*dd-(z2-r*r)*dd/zh;

  if (dd<0) printf("not from this location\n");
  

}
